#!/bin/sh
glilang=en


##  -------- Generate documentation for the Greenstone Librarian Interface --------

## ---- Check Javadoc exists ----
javadocpath=

# Some users may set the above line manually
if [ "x$javadocpath" = "x" ]; then

    # If it is set, use the JAVA_HOME environment variable
    if [ "x$JAVA_HOME" != "x" ]; then
	javadocpath="$JAVA_HOME/bin/javadoc"

    # Check if Javadoc is on the search path
    else
	javadocpath=`which javadoc 2> /dev/null`
    fi
fi

# Check that a Javadoc executable has been found
if [ "$glilang" = "es" ]; then
    echo "Revisando Javadoc: $javadocpath"
elif [ "$glilang" = "fr" ]; then
    echo "Vrification de Javadoc: $javadocpath"
elif [ "$glilang" = "ru" ]; then
    echo " Javadoc: $javadocpath"
else
    echo "Checking Javadoc: $javadocpath"
fi
if [ ! -x "$javadocpath" ]; then
    echo
    if [ "$glilang" = "es" ]; then
	echo "No se pudo localizar una versin apropiada de Javadoc. Usted deber "
        echo "instalar un Kit de Desarrollo de Software Java (versin 1.4 o "
        echo "superior) antes de generar la documentacin para la Interfaz de la "
        echo "Biblioteca Digital Greenstone."
    elif [ "$glilang" = "fr" ]; then
	echo "Une version approprie de Javadoc n'a pas pu tre localise."
	echo "Vous devez installer un Kit de Dveloppement Java (version 1.4 ou"
	echo "suprieure) avant de produire la documentation de Greenstone"
	echo "Librarian Interface."
    elif [ "$glilang" = "ru" ]; then
	echo "      Javadoc."
	echo "   Java Development Kit ( 1.4  ) "
	echo ",      "
	echo " Greenstone."
    else
	echo "Failed to locate an appropriate version of Javadoc. You must install a"
	echo "Java Development Kit (version 1.4 or greater) before generating the"
	echo "documentation for the Greenstone Librarian Interface."
    fi
    exit 1
fi


## ---- Document the GLI ----
echo
if [ "$glilang" = "es" ]; then
    echo "Generando documentacin para la Interfaz de la Biblioteca Digital"
    echo "Greenstone..."
elif [ "$glilang" = "fr" ]; then
    echo "Production de documentation pour la Greenstone Librarian Interface"
elif [ "$glilang" = "ru" ]; then
    echo "     Greenstone"
else
    echo "Generating documentation for the Greenstone Librarian Interface..."
fi

javadoc -classpath classes/:GLI.jar:lib/apache.jar:lib/qfslib.jar -sourcepath src/ -source 1.4 -author -breakiterator -d docs/ -group "Greenstone Librarian Interface" org.greenstone.gatherer:org.greenstone.gatherer.cdm:org.greenstone.gatherer.collection:org.greenstone.gatherer.feedback:org.greenstone.gatherer.file:org.greenstone.gatherer.gems:org.greenstone.gatherer.gui:org.greenstone.gatherer.gui.metaaudit:org.greenstone.gatherer.gui.tree:org.greenstone.gatherer.metadata:org.greenstone.gatherer.shell:org.greenstone.gatherer.util -private -splitindex -version   org.greenstone.gatherer org.greenstone.gatherer.cdm org.greenstone.gatherer.collection org.greenstone.gatherer.feedback org.greenstone.gatherer.file org.greenstone.gatherer.gems org.greenstone.gatherer.gui org.greenstone.gatherer.gui.metaaudit org.greenstone.gatherer.gui.tree org.greenstone.gatherer.metadata org.greenstone.gatherer.shell org.greenstone.gatherer.util

if [ "$glilang" = "es" ]; then
    echo "Hecho!"
elif [ "$glilang" = "fr" ]; then
    echo "Termin!"
elif [ "$glilang" = "ru" ]; then
    echo "!"
else
    echo "Done!"
fi