 /**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at th * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: Shaoqun Wu, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 2006 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *############################################## ##########################
 */
package org.greenstone.gatherer.gems;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import java.lang.String;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;

import java.io.File;

import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.DebugStream;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.GetOpt;
import org.greenstone.gatherer.gui.GComboBox;
import org.greenstone.gatherer.gui.GLIButton;
import org.greenstone.gatherer.gui.HelpFrame;
import org.greenstone.gatherer.gui.ModalDialog;
import org.greenstone.gatherer.gui.NonWhitespaceField;
import org.greenstone.gatherer.util.Codec;
import org.greenstone.gatherer.util.JarTools;
import org.greenstone.gatherer.util.Utility;
import org.greenstone.gatherer.util.XMLTools;
import org.greenstone.gatherer.Dictionary;
import org.w3c.dom.*;


/* 
 * @author Shaoqun Wu, Greenstone Digital Library, University of Waikato
 * @version 2.4
 */ 
public class GEMS
    extends JFrame implements WindowFocusListener 
{
 
    static final private Dimension SIZE = new Dimension(800,550);
    static final private Dimension TREE_SIZE = new Dimension(250,550);
    static final private Dimension TABLE_SIZE = new Dimension(550,200);

    private GEMS self = null;

    private OpenMetadataSetPrompt open_prompt;
    private DeleteMetadataSetPrompt delete_prompt;
    private NewMetadataSetPrompt new_prompt;   

     
    /** Um, the size of the screen I'd guess. */
    private Dimension screen_size = null;
    
    private MetadataSetManager msm;

    private MetadataElementTable meta_element_table; 
    private LangDependElementTable lang_element_table; 

    private MetadataSetTree  meta_set_tree;     

    private  MetadataSetModel metadata_set_model;

    private boolean stand_alone = true;

    private ArrayList listeners;

    static public void main(String[] args)
    {
	//TODO: add an option to open a paticular metadata set

	// Parse arguments
	GetOpt go = new GetOpt(args);

	if (go.debug) {
	    DebugStream.enableDebugging();

	    Calendar now = Calendar.getInstance();
	    String debug_file_path = "debug" + now.get(Calendar.DATE) + "-" + now.get(Calendar.MONTH) + "-" + now.get(Calendar.YEAR) + ".txt";
	    
	    // Debug file is created in the GLI directory
	    DebugStream.println("Debug file path: " + debug_file_path);
	    DebugStream.setDebugFile(debug_file_path);   

	}
       	new GEMS(go.gsdl_path,go.metadata_path,true,go.new_set);
    }


    /** Constructor.
     * @param set a MetadataSet that should be initially selected
     * @param action a systematic action that should be performed
     */
    public GEMS(String gsdl_path, String metadata_path, boolean standalone, boolean new_set)
    {
	self = this;
	JarTools.initialise(this);        
	screen_size = Configuration.screen_size;
        msm = new MetadataSetManager(gsdl_path);       
        stand_alone = standalone;
	listeners = new ArrayList();

        addWindowListener(new WindowAdapter(){
		public void windowClosing(WindowEvent e){
		    metadata_set_model.save(true);
		    if (stand_alone){
			System.exit(0);
		    }
		    else{
			notifyListeners();
			setVisible(false);
		    }
		}
	    });
	
	setSize(SIZE);
	setTitle(Dictionary.get("GEMS.Title"));
	setJMenuBar(new GEMSMenuBar());

	addWindowFocusListener(this);

        metadata_set_model = new MetadataSetModel(msm);
        msm.setMetadataSetModel(metadata_set_model);
	meta_set_tree = new MetadataSetTree();
        metadata_set_model.addObserver(meta_set_tree);

	meta_element_table = new MetadataElementTable();
	MetadataElementModel  mem= new MetadataElementModel();
        mem.addMetadataElementListener((MetadataElementListener)meta_element_table);
	meta_element_table.addAttributeListener(metadata_set_model);

	lang_element_table = new LangDependElementTable();
        mem.addMetadataElementListener((MetadataElementListener)lang_element_table);
	lang_element_table.addAttributeListener(metadata_set_model);

      	MetadataSetInfo msti= new MetadataSetInfo();
        msti.addMetadataSetListener((MetadataSetListener)meta_element_table);
        msti.addMetadataSetListener((MetadataSetListener)lang_element_table);
        msti.setMetadataSetModel(metadata_set_model);
        
        open_prompt = new OpenMetadataSetPrompt(self,msm);
        open_prompt.addMetadataSetListener((MetadataSetListener)metadata_set_model);
        open_prompt.addMetadataSetListener((MetadataSetListener)meta_element_table);        
	open_prompt.addMetadataSetListener((MetadataSetListener)lang_element_table); 	
	
        delete_prompt = new DeleteMetadataSetPrompt(self,msm);
        delete_prompt.addMetadataSetListener((MetadataSetListener)metadata_set_model); 
        delete_prompt.addMetadataSetListener((MetadataSetListener)meta_element_table);
	delete_prompt.addMetadataSetListener((MetadataSetListener)lang_element_table);
	
        new_prompt = new NewMetadataSetPrompt(self,msm);
        new_prompt.addMetadataSetListener((MetadataSetListener)metadata_set_model); 
        new_prompt.addMetadataSetListener((MetadataSetListener)meta_element_table);
        new_prompt.addMetadataSetListener((MetadataSetListener)lang_element_table);
  

        // load the initial metadataset
        if (metadata_path !=null && !metadata_path.equals("")){
	    open_prompt.openMetadataSet(metadata_path);
	}
	else{
	    if (new_set) new_prompt.display();
	}

        JScrollPane treePane = new JScrollPane(meta_set_tree);
        treePane.setPreferredSize(TREE_SIZE);

	JScrollPane tablePane = new JScrollPane(meta_element_table);
        tablePane.setPreferredSize(TABLE_SIZE);

        JScrollPane langTablePane =  new JScrollPane(lang_element_table);


        JPanel buttonPane = new JPanel(new GridLayout(1,2));
        
        JButton arrow_up_button = new GLIButton(Dictionary.get("GEMS.Move_Up"));
	arrow_up_button.setIcon(JarTools.getImage("arrow-up.gif"));

        JButton arrow_down_button = new GLIButton(Dictionary.get("GEMS.Move_Down"));
	arrow_down_button.setIcon(JarTools.getImage("arrow-down.gif")); 
        
        arrow_up_button.setActionCommand(GEMSConstants.MOVE_UP);
	arrow_up_button.addActionListener(meta_set_tree);
        arrow_down_button.addActionListener(meta_set_tree);
	arrow_down_button.setActionCommand(GEMSConstants.MOVE_DOWN);
	
	buttonPane.add(arrow_up_button);
        buttonPane.add(arrow_down_button);
        
        JPanel leftPane =  new JPanel(new BorderLayout());          
        leftPane.add(treePane,BorderLayout.CENTER);  
	leftPane.add(buttonPane,BorderLayout.SOUTH);  

	// Initialise popup menus
	createAttributePopupMenu();
	createElementPopupMenu();
	createLanguageDependAttributePopupMenu();
         
	JLabel  selected_language_label = new JLabel(Dictionary.get("GEMS.SelecedLanguage"));
	selected_language_label.setOpaque(true);
        
        JComboBox language_combo = new JComboBox(new Vector(msm.getLanguageList()));
	language_combo.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e){
		    JComboBox language_combo = (JComboBox) e.getSource();
		    String langInfo = (String)language_combo.getSelectedItem(); 
                    String lang = langInfo.split("\\s")[0];     		    
		    lang_element_table.languageSelected(lang);
		}

	    });      

       	JLabel  language_label = new JLabel(Dictionary.get("GEMS.LanguageDependent"));
	language_label.setOpaque(true);
      
         	
	JPanel selectLangPane = new JPanel(new BorderLayout(5,5));
        selectLangPane.add(selected_language_label,BorderLayout.WEST);
        selectLangPane.add(language_combo, BorderLayout.CENTER);   


        JPanel languagePane =  new JPanel(new BorderLayout(5,5));
	languagePane.add(language_label,BorderLayout.NORTH);  
        languagePane.add(selectLangPane,BorderLayout.CENTER);        


        JPanel languageAttributePane =  new JPanel(new BorderLayout(5,5));
	languageAttributePane.add(languagePane,BorderLayout.NORTH);  
        languageAttributePane.add(langTablePane,BorderLayout.CENTER);        


        JPanel rightPane =  new JPanel(new BorderLayout(5,5));
	rightPane.add(tablePane,BorderLayout.NORTH);  
        rightPane.add(languageAttributePane,BorderLayout.CENTER); 

	JSplitPane splitPane =  new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,leftPane,rightPane); 	
                                    
	getContentPane().add(splitPane,BorderLayout.CENTER);

	setLocation((screen_size.width - SIZE.width) / 2, (screen_size.height - SIZE.height) / 2);
	if (stand_alone)
	    setVisible(true);
    }

    public void displayMetadataSet(String metadata_path){
	open_prompt.openMetadataSet(metadata_path);
	setVisible(true);
    }   

    public void newMetadataSet(){
	new_prompt.display();
	setVisible(true);
    }

    public void addGEMSListener(GEMSListener listener){
	listeners.add(listener);
    }  

    public void removeGEMSListener(GEMSListener listener){
	listeners.remove(listener);
    }  

    public void notifyListeners(){
	for (int i=0;i<listeners.size();i++){
	    GEMSListener listener = (GEMSListener) listeners.get(i); 
	    listener.gemsIsClosed();
	}
    }


    public void exit(){
	//clean up things
	MetadataSetInfo msti= new MetadataSetInfo();
	msti.removeAllMetadataSetListeners();
	MetadataElementModel mem= new MetadataElementModel();
	mem.removeAllMetadataElementListeners();
    }


   public void createAttributePopupMenu() {
        JMenuItem menuItem;

        //Create the popup menu.
        JPopupMenu popup = new JPopupMenu();
        menuItem = new JMenuItem(Dictionary.get("GEMS.Popup.AddAttribute"));
        menuItem.addActionListener(meta_element_table);
        menuItem.setActionCommand(GEMSConstants.ADD_ATTRIBUTE);
        popup.add(menuItem);
        menuItem = new JMenuItem(Dictionary.get("GEMS.Popup.DeleteAttribute"));
        menuItem.addActionListener(meta_element_table);
	menuItem.setActionCommand(GEMSConstants.DELETE_ATTRIBUTE);
        popup.add(menuItem);

        //Add listener to the text area so the popup menu can come up.
        MouseListener popupListener = new PopupListener(popup);
        meta_element_table.addMouseListener(popupListener);
               
    }

  public void createLanguageDependAttributePopupMenu() {
        JMenuItem menuItem;

        //Create the popup menu.
        JPopupMenu popup = new JPopupMenu();
        //menuItem = new JMenuItem(Dictionary.get("GEMS.Popup.AddAttribute"));
        //menuItem.addActionListener(lang_element_table);
        //menuItem.setActionCommand(GEMSConstants.ADD_ATTRIBUTE);
        //popup.add(menuItem);
        menuItem = new JMenuItem(Dictionary.get("GEMS.Popup.DeleteAttribute"));
        menuItem.addActionListener(lang_element_table);
	menuItem.setActionCommand(GEMSConstants.DELETE_ATTRIBUTE);
        popup.add(menuItem);

        //Add listener to the text area so the popup menu can come up.
        MouseListener popupListener = new PopupListener(popup);
        lang_element_table.addMouseListener(popupListener);
         
    }



      public void createElementPopupMenu() {
        JMenuItem menuItem;
	//Create the popup menu.
        JPopupMenu popup = new JPopupMenu();
        menuItem = new JMenuItem(Dictionary.get("GEMS.Popup.AddElement"));
        menuItem.addActionListener(meta_set_tree);
        menuItem.setActionCommand(GEMSConstants.ADD_ELEMENT);
        popup.add(menuItem);
        menuItem = new JMenuItem(Dictionary.get("GEMS.Popup.DeleteElement"));
        menuItem.addActionListener(meta_set_tree);
	menuItem.setActionCommand(GEMSConstants.DELETE_ELEMENT);
        popup.add(menuItem);
	
        //Add listener to the text area so the popup menu can come up.
        MouseListener popupListener = new PopupListener(popup);
        meta_set_tree.addMouseListener(popupListener); 
        
      }
 
    public void windowGainedFocus(WindowEvent e) {
	//System.out.println("windowGainedFocus.");
    }

    public void windowLostFocus(WindowEvent e) {
        //System.out.println("windowLostFocus.");
          
	//save stuff
	
    }

   
    private class GEMSMenuBar
	extends JMenuBar
	implements ActionListener
    {
	private JMenu file = null;
	private JMenu edit = null;

	public JMenuItem file_exit   = null;
	public JMenuItem file_new    = null;
	public JMenuItem file_open   = null;
	public JMenuItem file_save   = null;
	public JMenuItem edit_copy   = null;
	public JMenuItem edit_cut    = null;
	public JMenuItem edit_paste  = null;
	public JMenuItem help_help   = null;
        public JMenuItem file_delete = null;
        //public JMenuItem file_preferences = null;
	public GEMSMenuBar()
	{
	    file = new JMenu();
	    file.setText(Dictionary.get("Menu.File"));
	    
	    file_exit = new JMenuItem(Dictionary.get("Menu.File_Close"));
	    file_exit.addActionListener(this);
	    
	    file_new = new JMenuItem(Dictionary.get("Menu.File_New"));
	    file_new.addActionListener(this);
	    
	    file_open = new JMenuItem(Dictionary.get("Menu.File_Open"));
	    file_open.addActionListener(this);
	    
	    file_save = new JMenuItem(Dictionary.get("Menu.File_Save"));
	    file_save.addActionListener(this);
   
            file_delete = new JMenuItem(Dictionary.get("Menu.File_Delete"));
	    file_delete.addActionListener(this);
	                
            //file_preferences = new JMenuItem(Dictionary.get("Menu.File_Options"));
	    //file_preferences.addActionListener(this);
	    
	    // Layout (file menu)
	    file.add(file_new);
	    file.add(file_open);
	    file.add(file_save);
            file.add(file_delete);
	    file.add(new JSeparator());
            //file.add(file_preferences);
            file.add(new JSeparator());
	    file.add(file_exit);
            
	    // Edit menu
	    edit = new JMenu();
	    edit.setText(Dictionary.get("Menu.Edit"));
	    
	    edit_cut = new JMenuItem(Dictionary.get("Menu.Edit_Cut"));
	    edit_cut.addActionListener(this);
	    
	    edit_copy = new JMenuItem(Dictionary.get("Menu.Edit_Copy"));
	    edit_copy.addActionListener(this);
	    
	    edit_paste = new JMenuItem(Dictionary.get("Menu.Edit_Paste"));
	    edit_paste.addActionListener(this);
	    
	    // Layout (edit menu)
	    edit.add(edit_cut);
	    edit.add(edit_copy);
	    edit.add(edit_paste);

	
	    // Layout (menu bar)
	    this.add(file);
	    this.add(Box.createHorizontalStrut(15));
	    this.add(edit);
	    this.add(Box.createHorizontalGlue());
	    //this.add(help);
	}


	public void actionPerformed(ActionEvent event)
	{
	    Object event_source = event.getSource();

	    // File -> New
	    if (event_source == file_new) {
		new_prompt.display();   
 		return;
	    }

	    // File -> Open
	    if (event_source == file_open) {               
		open_prompt.display();
		return;
	    }

              // File -> Delete
	    if (event_source == file_delete) {               
                delete_prompt.display();
		return;
	    }


	    // File -> Save
	    if (event_source == file_save) {
		metadata_set_model.save(false);
		return;
	    }        

	    // File -> Exit
	    if (event_source == file_exit) {
               	metadata_set_model.save(true); 
		if (stand_alone){
		    System.exit(0);
		}
		else{
		    notifyListeners();
		    setVisible(false);
		}
	    }

	    /// File -> Preferences
	    // if(event_source == file_preferences){
		// GEMSPreferences GemsPreferences = new GEMSPreferences();  
            //}

	    // Edit -> Cut
	    if (event_source == edit_cut) {
		try {
		    KeyboardFocusManager kfm = KeyboardFocusManager.getCurrentKeyboardFocusManager();
		    // Get the component with selected text as a JTextComponent
		    JTextComponent text = (JTextComponent) kfm.getPermanentFocusOwner();
		    // Cut the text to the clipboard
		    text.cut();
		}
		catch (ClassCastException cce) {
		    // If the component is not a text component ignore the cut command
		    DebugStream.println(cce.toString());
		}
		return;
	    }

	    // Edit -> Copy
	    if (event_source == edit_copy) {
		try {
		    KeyboardFocusManager kfm = KeyboardFocusManager.getCurrentKeyboardFocusManager();
		    // Get the component with selected text as a JTextComponent
		    JTextComponent text = (JTextComponent) kfm.getPermanentFocusOwner();//getFocusOwner();
		    text.copy();
		}
		catch (Exception cce) {
		    // If the component is not a text component ignore the copy command
		    DebugStream.println(cce.toString());
		}
		return;
	    }

	    // Edit -> Paste
	    if (event_source == edit_paste) {
		try {
		    KeyboardFocusManager kfm = KeyboardFocusManager.getCurrentKeyboardFocusManager();
		    // Get the component with selected text as a JTextComponent
		    JTextComponent text = (JTextComponent) kfm.getPermanentFocusOwner();
		    // Cut the text to the clipboard
		    text.paste();
		}
		catch (ClassCastException cce) {
		    // If the component is not a text component ignore the paste command
		    DebugStream.println(cce.toString());
		}
		return;
	    }
	  
	}
    }


   
 class PopupListener extends MouseAdapter {
        JPopupMenu popup;

        PopupListener(JPopupMenu popupMenu) {
            popup = popupMenu;
        }

        public void mousePressed(MouseEvent e) {
            maybeShowPopup(e);
        }

        public void mouseReleased(MouseEvent e) {
            maybeShowPopup(e);
        }

        private void maybeShowPopup(MouseEvent e) {
            Component source = (Component)e.getSource();
            boolean show = false;

	    if (source instanceof JTable){
                show = true;
		JTable table = (JTable)source;
		JMenuItem item = null;
                if (table.getSelectedRow() !=-1){
		    
		    if(popup.getComponentCount() >1){
			item = (JMenuItem)popup.getComponent(1);
		    }
		    else{
			item = (JMenuItem)popup.getComponent(0);
		    } 
		 
		    item.setEnabled(true);
		}
		else{//disable the delete item if no row is selected
		    if(popup.getComponentCount() >1){
			item = (JMenuItem)popup.getComponent(1);
		    }
		    else{
			item = (JMenuItem)popup.getComponent(0);
		    }
		    item.setEnabled(false);
		}
	    }              

            if (source instanceof JTree){
		
		JTree tree = (JTree)source;
		TreePath tse = tree.getSelectionPath();
		if (tse ==null) return;
		DefaultMutableTreeNode snode= (DefaultMutableTreeNode)tse.getLastPathComponent();
			  
		Object data = snode.getUserObject();
                // don't allow the root element deletion	      
		if (data instanceof MetadataElementModel){                
		    JMenuItem item = (JMenuItem)popup.getComponent(1);
                    item.setEnabled(true);
		}
		else{
		    JMenuItem item = (JMenuItem)popup.getComponent(1);
                    item.setEnabled(false);
		}
                
		
		// don't allow to add elements two levels deep	      
		if (tse.getPath().length > 2){                
		    JMenuItem item = (JMenuItem)popup.getComponent(0);
                    item.setEnabled(false);
		}
		else{
		    JMenuItem item = (JMenuItem)popup.getComponent(0);
                    item.setEnabled(true);
		}


		show = true;
	    }              

            

            if (e.isPopupTrigger() && show) {
                popup.show(e.getComponent(),
                           e.getX(), e.getY());
            }
        }
   }
}
