/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: Shaoqun Wu, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 2006 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gems;

import org.w3c.dom.Element;
 
import javax.swing.table.*; 
import javax.swing.JTable;
import javax.swing.JLabel; 

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ArrayList; 
import java.util.HashMap;
import java.util.Iterator;
import javax.swing.JOptionPane;
import java.awt.Rectangle;
import java.awt.Component;
import java.awt.Color;

import javax.swing.DefaultCellEditor;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import org.greenstone.gatherer.Dictionary;

/**
 * @author Shaoqun Wu, Greenstone Digital Library, University of Waikato
 * @version 2.4
 */
public class MetadataElementTable 
    extends JTable implements MetadataElementListener, MetadataSetListener, ActionListener {
    int DISPLAY_INFO = 0;
  
    int DISPLAY_ELEMENT = 1;
    
    int state = -1; 

    JTable table; 

    private ArrayList listeners = new ArrayList();
    private JTextField metadata_value_text_field = new JTextField();

    public MetadataElementTable(){
	super();       
        setRowHeight(20);
        setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

    	metadata_value_text_field.setBorder(null);
	DefaultCellEditor cellEditor = new DefaultCellEditor(metadata_value_text_field);
	cellEditor.setClickCountToStart(1);
	setDefaultEditor(String.class,cellEditor);   
        table = this;
	setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      }


    public void actionPerformed(ActionEvent e){
	String command = e.getActionCommand();
   
        if (command.equals(GEMSConstants.ADD_ATTRIBUTE)){
	    addAttribute();
 	    return ;
	}
       
       if (command.equals(GEMSConstants.DELETE_ATTRIBUTE)){
           deleteAttribute();
	}   
           
    }

    private void addAttribute(){
	if (state == DISPLAY_ELEMENT){
	    MetadataElementTableModel model = (MetadataElementTableModel)getModel();
            model.addNewRow();
	}                           
	else{
	    MetadataSetInfoTableModel model = (MetadataSetInfoTableModel)getModel();
	    model.addNewRow();     
	}
         
    }

    private void deleteAttribute(){
	int index = getSelectedRow();
      	if (state == DISPLAY_ELEMENT){
	    MetadataElementTableModel model = (MetadataElementTableModel)getModel();
	    model.deleteRow(index);
	}                           
	else{

	    MetadataSetInfoTableModel model = (MetadataSetInfoTableModel)getModel();
	    model.deleteRow(index);  
	}   
    }

    public void metadataElementChanged(MetadataElementEvent mee){
	state = DISPLAY_ELEMENT;
	MetadataElementModel model = mee.getMetadataElementModel();      
        if (model == null){ //this model has been deleted
	    setModel(new DefaultTableModel());
            return;
	} 
	populateMetadataElementTable(model);
    }

    public void metadataSetChanged(MetadataSetEvent mse){
        state = DISPLAY_INFO;
        MetadataSetInfo info = mse.getMetadataSetInfo();
	if (info == null){
	    setModel(new DefaultTableModel());
	    return;
	}
        populateMetadataElementTable(info);
    }

  
    private void populateMetadataElementTable(MetadataElementModel model){
	MetadataElementTableModel table_model = new MetadataElementTableModel(model);
        setModel(table_model);
	getColumnModel().getColumn(0).setPreferredWidth(100);
        getColumnModel().getColumn(1).setPreferredWidth(415);
    }
      
  

    private void populateMetadataElementTable(MetadataSetInfo info){
	MetadataSetInfoTableModel model = new MetadataSetInfoTableModel(info);
	setModel(model);
	getColumnModel().getColumn(0).setPreferredWidth(100);
        getColumnModel().getColumn(1).setPreferredWidth(415);         
    } 


    public void addAttributeListener(AttributeListener al){
	if (!listeners.contains(al)){
	    listeners.add(al);    
	}
    }

    public void removeAttributeListener(AttributeListener al){
	listeners.remove(al);   
    }

    private void notifyListeners(Attribute attr){
	AttributeEvent ae = new AttributeEvent(attr);
	for(int i=0;i<listeners.size();i++){
	    AttributeListener al = (AttributeListener) listeners.get(i);
	    al.attributeChanged(ae);
	    
	}
    }

    class MetadataElementTableModel extends AbstractTableModel{
	MetadataElementModel element_model;
        ArrayList attributes;
	String[] columnNames= new String[]{"name","value"};    

	
	public MetadataElementTableModel(MetadataElementModel model){
	    element_model = model;
	    attributes = element_model.getAttributes();            
	} 
	
	public  int getRowCount(){
	    return attributes.size() + 1; // one more for the name of the element
	}
       
	public String getColumnName(int column){
	    return columnNames[column];
	}
	
	public Class getColumnClass(int columnIndex){
	    return String.class;
	}    
	
	public boolean isCellEditable(int row, int col) {
	    if (row==0 && (col ==0 || col==1)) return false;
            if (row > 0){
		Attribute attr = (Attribute)attributes.get(row-1);
		if (attr.isRequired()) return false; 
	    }
	
	    return true;	 
	}
	
	public int getColumnCount(){
	    return columnNames.length;
	}
       
	public  Object getValueAt(int row, int column){
	    if (row == 0 && column == 0 ) return "<html><body Color=blue>name</body></html>";
            if (row == 0 && column == 1 ) return element_model.getName();
	    if (row <= attributes.size() && attributes.size()>0){
		Attribute attr = (Attribute)attributes.get(row-1);
		if (column == 0) return attr.getName();
		if (column == 1) return attr.getValue();
	    }
	    return null; 	    	    
	}

        public void setValueAt(Object value, int row, int column){    
	    // update the element name on tree
	     if (row == 0 && column == 1) {
		 if (((String)value).trim().equals("")){
		     JOptionPane.showMessageDialog(null,Dictionary.get("GEMS.Attribute_Edition_Error_Message"), Dictionary.get("GEMS.Attribute_Edition_Error"),JOptionPane.ERROR_MESSAGE);
		     return;
		 } 
		 element_model.setName((String)value);
                 return;
	     }
	     
            if (row <= attributes.size()  && attributes.size()>0 ){
		Attribute attr = (Attribute)attributes.get(row-1);
		if (attr.isRequired() && ((String)value).trim().equals("") && !getValueAt(row,column).equals("")) {
		    JOptionPane.showMessageDialog(null,Dictionary.get("GEMS.Attribute_Edition_Error_Message"), Dictionary.get("GEMS.Attribute_Edition_Error"),JOptionPane.ERROR_MESSAGE);
		    return; 
		}

		if (column == 0) attr.setName((String)value);
                if (column == 1) attr.setValue((String)value);
		notifyListeners(attr);
	    }

	}

	public void addNewRow(){
	    Attribute attr = new Attribute();
            attributes.add(attr);
            fireTableDataChanged(); 
	    table.getSelectionModel().setSelectionInterval(attributes.size()-1,attributes.size()-1);
	    Rectangle rect = table.getCellRect(attributes.size()-1, 0, true);
	    table.scrollRectToVisible(rect);             
	}

	public void deleteRow(int index){           
	    index--; //the first row is the name element, 
            if (index < attributes.size() && index >=0){
	        Attribute attr = (Attribute)attributes.get(index);          
		if (attr.isRequired()){
		    JOptionPane.showMessageDialog(null,Dictionary.get("GEMS.Attribute_Deletion_Error_Message"), Dictionary.get("GEMS.Attribute_Deletion_Error"),JOptionPane.ERROR_MESSAGE);
		    return;
		}

		int result = JOptionPane.showOptionDialog(null, Dictionary.get("GEMS.Confirm_Removal", Dictionary.get("GEMS.Attribute") + " " +  Dictionary.get("GEMS.Cannot_Undo")), Dictionary.get("GEMS.Confirm_Removal_Title"), JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,null,GEMSConstants.DIALOG_OPTIONS,GEMSConstants.DIALOG_OPTIONS[0] );
         	
		if (result !=JOptionPane.OK_OPTION) return;
		notifyListeners(attr);
		attributes.remove(index);
		fireTableDataChanged();  
		return;
	    }
            	         
	}
       
    }


    class MetadataSetInfoTableModel extends AbstractTableModel{
	MetadataSetInfo metadata_info;
        ArrayList attributes;
         
	String[] columnNames= new String[]{"name","value"};    
	  
	public MetadataSetInfoTableModel(MetadataSetInfo info){
	    metadata_info = info;
	    attributes = info.getAttributes();
	    
	} 
	
	public  int getRowCount(){
	    return  attributes.size();
	}
       
	public String getColumnName(int column){
	    return columnNames[column];
	}
	
	public Class getColumnClass(int columnIndex){
	    return String.class;
	}    
	
	public boolean isCellEditable(int row, int col) {
            Attribute attr = (Attribute)attributes.get(row);	   
	    if (col == 0 && attr.isRequired()) return false;
	    return true;
	}
  
	public int getColumnCount(){
	    return columnNames.length;
	}
       
	public  Object getValueAt(int row, int column){
	    if (row < attributes.size()){
		Attribute attr = (Attribute)attributes.get(row);
		String name = attr.getName();
                if (attr.isRequired()){
                  name = "<html><body Color=blue>"+name+"</body></html>";
		}

		if (column ==0 )return name;
		return attr.getValue();	
	    }	
	    
	    return null;	    
	}

	public void setValueAt(Object value, int row, int column){         
	    if (row < attributes.size()){  
              	Attribute attr = (Attribute)attributes.get(row);
		if (attr.isRequired() && ((String)value).trim().equals("") && !getValueAt(row,column).equals("")) {
		    JOptionPane.showMessageDialog(null,Dictionary.get("GEMS.Attribute_Edition_Error_Message"), Dictionary.get("GEMS.Attribute_Edition_Error"),JOptionPane.ERROR_MESSAGE);
		    return; 
		}

	        if (column == 0 && ((String)value).trim().equals("") && !attr.getName().trim().equals("")){
		    JOptionPane.showMessageDialog(null,Dictionary.get("GEMS.Attribute_Name_Error_Message"), Dictionary.get("GEMS.Attribute_Name_Error"),JOptionPane.ERROR_MESSAGE);
		    return; 
		}

		//check namespace conflict
		if (column == 1
		    && !((String)value).trim().equals(attr.getValue().trim())
                     && attr.getName().trim().equals(GEMSConstants.NAMESPACE_ATTRIBUTE)
                     && metadata_info.checkNamespace(((String)value).trim()))
		{
			
		    int result = JOptionPane.showOptionDialog(null,Dictionary.get("GEMS.Namespace_Conflict_Message"), Dictionary.get("GEMS.Namespace_Conflict"),JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,null,GEMSConstants.DIALOG_OPTIONS,GEMSConstants.DIALOG_OPTIONS[0] );
		    
		    if (result !=JOptionPane.OK_OPTION) return;
		    
		}
                
		
		if (column == 0) attr.setName((String)value);
		else  attr.setValue((String)value);
             
		notifyListeners(attr);

		if (attr.getName().trim().equals(GEMSConstants.NAMESPACE_ATTRIBUTE))
		    {
			metadata_info.infoChanged();           
		}
	        
	   }
             
	}


       	public void addNewRow(){
	    Attribute attr = new Attribute();
            attributes.add(attr);	    
            fireTableDataChanged(); 
            table.getSelectionModel().setSelectionInterval(attributes.size()-1,attributes.size()-1);
	    Rectangle rect = table.getCellRect(attributes.size()-1, 0, true);
	    table.scrollRectToVisible(rect);              
	}

	public void deleteRow(int index){
           
            if (index < attributes.size() && index >=0){
                 Attribute attr = (Attribute)attributes.get(index);          
		 if (attr.isRequired()){
		    JOptionPane.showMessageDialog(null,Dictionary.get("GEMS.Attribute_Deletion_Error_Message"), Dictionary.get("GEMS.Attribute_Deletion_Error"),JOptionPane.ERROR_MESSAGE);
		    return;
		 }

		int result = JOptionPane.showOptionDialog(null, Dictionary.get("GEMS.Confirm_Removal", Dictionary.get("GEMS.Attribute") + " " +  Dictionary.get("GEMS.Cannot_Undo")), Dictionary.get("GEMS.Confirm_Removal_Title"), JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE,null, GEMSConstants.DIALOG_OPTIONS,GEMSConstants.DIALOG_OPTIONS[0]);
         	
		if (result !=JOptionPane.OK_OPTION) return;
		
		notifyListeners(attr);
		attributes.remove(index);
		fireTableDataChanged();  
		return;
	    }
	              
	}

    }


}






