/**********************************************************************
 *
 * pageaction.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include "pageaction.h"
#include "receptionist.h"
#include "fileutil.h"

pageaction::pageaction () {

  recpt = NULL;

  cgiarginfo arg_ainfo;
  arg_ainfo.shortname = "a";
  arg_ainfo.longname = "action";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "p";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);

  arg_ainfo.shortname = "p";
  arg_ainfo.longname = "page";
  arg_ainfo.multiplechar = true;
  arg_ainfo.defaultstatus = cgiarginfo::weak;
  arg_ainfo.argdefault = "home";
  arg_ainfo.savedarginfo = cgiarginfo::must;
  argsinfo.addarginfo (NULL, arg_ainfo);
}

pageaction::~pageaction () {
}

void pageaction::get_cgihead_info (cgiargsclass &args, response_t &response,
				   text_t &response_data, ostream &logout) {
  response = content;
  response_data = "text/html";
}

bool pageaction::do_action (cgiargsclass &args, displayclass &disp, 
			    outconvertclass &outconvert, ostream &textout, 
			    ostream &logout) {

  text_t &arg_p = args["p"];

  textout << outconvert << disp << ("_" + arg_p + ":header_\n")
	  << ("_" + arg_p + ":content_\n")
	  << ("_" + arg_p + ":footer_\n");

  return true;
}

void pageaction::define_internal_macros (displayclass &disp, cgiargsclass &args, 
					 ostream &logout) {

  text_t &arg_p = args["p"];

  if (arg_p == "homepref") {
    // set _languageoption_ and _encodingoption_
    set_language_encoding_macros(disp, args, logout);
  }
}

void pageaction::set_language_encoding_macros(displayclass &disp, cgiargsclass &args,
					      ostream &logout) {
  // _languageoption_
  // Create the "interface language" selection box for the preferences pages.
  text_t &arg_l = args["l"];
  const recptconf &configinfo = recpt->get_configinfo();
  // put languages in another map to sort them by longname
  text_tmap languages;
  languageinfo_tmap::const_iterator thislang = configinfo.languages.begin();
  languageinfo_tmap::const_iterator endlang = configinfo.languages.end();
  while (thislang != endlang) {
    languages[(*thislang).second.longname] = (*thislang).first;
    thislang++;
  }
  text_tmap::iterator tlang = languages.begin();
  text_tmap::iterator elang = languages.end();

  text_t languageoption;
  bool collection_specific = false;

  if (!collection_specific) {
    while (tlang != elang) {
      languageoption += "<option value=\"" + (*tlang).second + "\"";
      if ((*tlang).second == arg_l) languageoption += " selected";
      languageoption += ">" + (*tlang).first + "\n";
      tlang ++;
    }
  }

  if (!languageoption.empty()) {
    languageoption = "<select name=\"l\" onChange=\"updatel();\">\n" + languageoption;
    languageoption += "</select>\n";
    disp.setmacro ("languageoption", args["p"], languageoption);
  }

  // _encodingoption_
  // create the "encoding" selection box for the preferences page
  // if (configinfo.encodings.size() > 1) {
    text_t &arg_w = args["w"];
    text_t encodingoption;
    text_tmap::const_iterator thisenc = configinfo.encodings.begin();
    text_tmap::const_iterator endenc = configinfo.encodings.end();
    while (thisenc != endenc) {
      encodingoption += "<option value=\"" + (*thisenc).second + "\"";
      if ((*thisenc).second == arg_w) encodingoption += " selected";
      encodingoption += ">" + (*thisenc).first + "\n";
      thisenc ++;
    }

    encodingoption = "<select name=\"w\" onChange=\"updatew();\">\n" + encodingoption;
    encodingoption += "</select>\n";
    disp.setmacro ("encodingoption", args["p"], encodingoption);
    // }
}
