package org.greenstone.gsdl3.gs3build.doctypes;

import java.util.ArrayList;
import java.net.URL;
import org.w3c.dom.Element;

import org.greenstone.gsdl3.gs3build.CollectionManager;
import org.greenstone.gsdl3.gs3build.metadata.*;
import org.greenstone.gsdl3.gs3build.util.HTTPTools;
/**
   A simple base class for Recognisers
   A new recogniser should set preferred_mime_type, filename_extensions, and document_type in the constructor. 
   The DocumentFactory class needs to be modified to map the documentType to 
   the corresponding document
   And thats all for simple recognisers.
   More complicated recognisers can just implement from scratch RecogniserInterface
*/
abstract public class AbstractRecogniser implements RecogniserInterface
{

    DocumentList list_repository = null;
    String preferred_mime_type = "SET THIS IN THE CONCRETE CLASS"; 
    ArrayList filename_extensions = null;
    String document_type = "SET THIS IN THE CONCRETE CLASS";
    CollectionManager coll_manager = null;

    /** The constructor should set the variables 
     * preferredMimeType, filename_extensions and documentType
     */
    public AbstractRecogniser() {

    }
    
    /** set the collection manager */
    public void setCollectionManager(CollectionManager coll_man) {
	this.coll_manager = coll_man;
    }
    /** configure by default does nothing */
    public boolean configure(Element config_elem){
	return true;
    }
    public void setListRepository(DocumentList docList) {
	this.list_repository = docList;
    }

    protected boolean isAcceptedFilename(String filename) {
	if (filename_extensions == null) {
	    return false; // or should this be true??
	}
	for (int i=0; i<filename_extensions.size(); i++) {
	    if (filename.endsWith((String)filename_extensions.get(i))) {
		return true;
	    }
	}
	return false;
    }
    
    public boolean parseDocument(METSFile file)
    {
	String MIMEType = file.getMIMEType();
	if (MIMEType == null || 
	    MIMEType.equals(this.preferred_mime_type)) {
	    URL location = file.getLocation();
	    return this.parseDocument(location);
	}
	return false;
    }

    public boolean parseDocument(URL url)
    { 
	String filename = null;
	
	if (url.getProtocol().equals("file")) {
	    filename = url.getPath();
	}
	
	if (filename != null) {
	    if ( isAcceptedFilename(filename)) {
		
		System.out.println("Posting "+document_type+" Document " + filename);
		AbstractDocument doc = DocumentFactory.createDocument(document_type, url);
		this.list_repository.addDocument(doc);
		return true;
	    }
	} else {
	    // Get Mime type remotely, and then proceed if required
	    String mimeType = HTTPTools.getMIMEType(url);
	    
	    if (mimeType == this.preferred_mime_type) {
		System.out.println("Posting "+document_type+" Document " + url.toString());
		
		AbstractDocument doc = DocumentFactory.createDocument(document_type, url);
		this.list_repository.addDocument(doc);
		return true;
	    }
	}
	return false;
    }
}


