###########################################################################
#
# MARCXMLPlugout.pm -- the plugout module for MARC xml recored
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 2006 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package MARCXMLPlugout;

use strict;
no strict 'refs';

eval {require bytes};
use util;
use BasPlugout;

sub BEGIN {
    @MARCXMLPlugout::ISA = ('BasPlugout');
}

my $arguments = [
		 { 'name' => "group", 
		   'desc' => "{MARCXMLPlugout.group}",
		   'type' => "flag",
                   'deft' => "0",
		   'reqd' => "no",    
		   'hiddengli' => "no"}
                ];

my $options = { 'name'     => "MARCXMLPlugout",
		'desc'     => "{MARCXMLPlugout.desc}",
		'abstract' => "no",
		'inherits' => "yes",
	        'args'     => $arguments
                };

sub new {
    my ($class) = shift (@_);
    my ($plugoutlist, $inputargs,$hashArgOptLists) = @_;
    push(@$plugoutlist, $class);

    if(defined $arguments){ push(@{$hashArgOptLists->{"ArgList"}},@{$arguments});}
    if(defined $options) { push(@{$hashArgOptLists->{"OptList"}},$options)};

    my $self = (defined $hashArgOptLists)? new BasPlugout($plugoutlist,$inputargs,$hashArgOptLists): new BasPlugout($plugoutlist,$inputargs); 

         
   $self->{'buffered_output'} ="";

   return bless $self, $class;
}

sub saveas {
    my $self = shift (@_);
    my ($doc_obj,$doc_dir) = @_;
 
    if ($self->{'group'}){
        $self->{buffered_output} .= $doc_obj->get_top_metadata_list()."\n";  
        return;
    }
 
    my $output_dir = $self->get_output_dir();
    &util::mk_all_dir ($output_dir) unless -e $output_dir;
    
    my $working_dir = &util::filename_cat ($output_dir, $doc_dir);    
    &util::mk_all_dir ($working_dir) unless -e $working_dir;

    my $output_file = util::filename_cat ($working_dir, "marc.xml");

    $self->open_xslt_pipe($output_file,$self->{'xslt_file'});

    my $outhandler = $self->{'xslt_writer'}; 

    $self->output_xml_header($outhandler,"MARCXML");
    print $outhandler $doc_obj->get_top_metadata_list();
    $self->output_xml_footer($outhandler,"MARCXML");  
    $self->close_xslt_pipe(); 
              
}



sub close_group_output{
    my $self = shift (@_);
    
    return unless $self->{'group'} and  $self->{buffered_output};

    my $output_dir = $self->get_output_dir();
    &util::mk_all_dir ($output_dir) unless -e $output_dir;

    my $output_file = util::filename_cat($output_dir, "marc.xml");

    $self->open_xslt_pipe($output_file,$self->{'xslt_file'});

    my $outhandler = $self->{'xslt_writer'}; 

    $self->output_xml_header($outhandler,"MARCXML");
    print $outhandler $self->{buffered_output};
    $self->output_xml_footer($outhandler,"MARCXML");  
    $self->close_xslt_pipe();    
}

sub is_group{
     my $self = shift (@_);
     return $self->{'group'}; 
}

1;
