package org.nzdl.gsdl;

import java.io.*;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 *  Use the TraX interface to perform a transformation in the simplest manner possible
 *  (3 statements).
 */
public class ApplyXSLT
{

    public static final String DOC_START = new String ("<?DocStart?>"); 
    public static final String DOC_END = new String ("<?DocEnd?>"); 
    public static final String INPUT_END = new String ("<?Done?>"); 

    private final int BEFORE_READING = 0;
    private final int IS_READING = 1;
    private String xsl_file;

    public ApplyXSLT(String xsl_file)
    {
	this.xsl_file = xsl_file; 
    }

    private boolean process()
    {
	// Use System InputStream to receive piped data from the perl program
	InputStreamReader ir = new InputStreamReader(System.in);
	BufferedReader br = new BufferedReader(ir);
	
	int system_status = BEFORE_READING;
 	String a_doc = new String();
	String output_file = new String();

	try{
	    while (br.ready())
		{
                    
		    String this_line = br.readLine();
                     
                  	    
		    if(system_status == BEFORE_READING)
			{
			    
			    if(this_line.compareTo(DOC_START) == 0)
				{
				    output_file = br.readLine(); // read the next line as the output file name
				    system_status = IS_READING;
				    a_doc = "";
				}
			    else if(this_line.compareTo(INPUT_END) == 0)
				{
				    return true;
				}
			    else
				{
				     
				    System.err.println("Undefined process status:" + this_line);
				    system_status = BEFORE_READING;
				}
			    
			}
		    else if(system_status == IS_READING)
			{
			    if(this_line.compareTo(DOC_END) == 0)
				{
				   
				    if(translateXML(a_doc,xsl_file,output_file) == false)
					{
					    System.err.println("Translation Failed!!");
					    return false;
					}
				    system_status = BEFORE_READING;
				   
				}
			    else
				{
				       a_doc = a_doc + this_line + "\n";
                                       
				}
			}
		    else
			{
			  
			    System.err.println ("Undefined system status in ApplyXSLT.java main().");
			    System.exit(-1);
			}
		        
		}
	}catch (Exception e)
	    {
		System.err.println("Receiving piped data error!" + e.toString());
	    }
				
	return false;
    }


    private boolean translateXML(String full_doc, String xsl_file, String output_file) 
	throws IOException,TransformerException, TransformerConfigurationException, FileNotFoundException
    {
       
 	StringReader str = new StringReader(full_doc) ;
	
   	TransformerFactory tFactory = TransformerFactory.newInstance();
   	Transformer transformer = tFactory.newTransformer(new StreamSource(xsl_file));
   	transformer.transform(new StreamSource(str), new StreamResult(new FileOutputStream(output_file)));
	return true;
    }

    public static void main(String[] args) 
    { 
	// Checking Arguments
	if(args.length != 1)
	    {
		System.err.println("Must provide a XSL file !");
		System.exit(-1);
	    }
	ApplyXSLT core = new ApplyXSLT(args[0]);

	if(core.process())
	    {
		//System.err.println("System Message: Process Completed");
	    }
	else
	    {
		//System.err.println("System Message: Process Incompleted");
	    }
    }
}


